/* options.h 
	vi:se ts=3 sw=3:
 */
/* $Id: options.h,v 4.1 1994/01/12 16:11:07 espie Exp espie $ */
/* $Log: options.h,v $
 * Revision 4.1  1994/01/12  16:11:07  espie
 * Last minute changes.
 *
 * Revision 4.0  1994/01/11  17:51:16  espie
 * No major changes.
 *
 * Revision 1.1  1993/12/26  00:55:53  Espie
 * Initial revision
 *
 * Revision 1.1  1993/12/04  16:12:50  espie
 * Initial revision
 * */

#define BAD_OPTION (-1)
#define OPT_HELP 0
#define OPT_QUIET 1
#define OPT_PICKY 2
#define OPT_TOLERANT 3
#define OPT_NEWONLY 4
#define OPT_OLDONLY 5
#define OPT_BOTH 6
#define OPT_MONO 7
#define OPT_STEREO 8
#define OPT_VERBOSE 9
#define OPT_FREQUENCY 10
#define OPT_OVERSAMPLE 11
#define OPT_TRANSPOSE 12
#define OPT_REPEATS 13
#define OPT_SPEED 14
#define OPT_MIX 15
#define OPT_START 16
#define OPT_CUT 17
#define OPT_ADD 18
#define OPT_SHOW 19
#define OPT_SYNC 20
#define OPT_SPEAKER 21
#define OPT_VOLUME 22

